@@ -108,8 +108,10 @@ isthisnonword(MMIOT *f, int i)
 
 
 /* return/set the current cursor position
+ * (when setting the current cursor position we also need to flush the
+ * last character written cache)
  */
-#define mmiotseek(f,x)	(f->isp = x)
+#define mmiotseek(f,x)	((f->isp = x), (f->last = 0))
 #define mmiottell(f)	(f->isp)
 
 
@@ -224,6 +226,11 @@ ___mkd_reparse(char *bfr, int size, int flags, MMIOT *f, char *esc)
     ___mkd_emblock(&sub);
     
     Qwrite(T(sub.out), S(sub.out), f);
+    /* inherit the last character printed from the reparsed
+     * text;  this way superscripts can work when they're
+     * applied to something embedded in a link
+     */
+    f->last = sub.last;
 
     ___mkd_freemmiot(&sub, f->footnotes);
 }
@@ -637,6 +644,32 @@ extra_linky(MMIOT *f, Cstring text, Footnote *ref)
 } /* extra_linky */
 
 
+
+/* check a url (or url fragment to see that it begins with a known good
+ * protocol (or no protocol at all)
+ */
+static int
+safelink(Cstring link)
+{
+    char *p, *colon;
+
+    if ( T(link) == 0 )	/* no link; safe */
+	return 1;
+
+    p = T(link);
+    if ( (colon = memchr(p, ':', S(link))) == 0 )
+	return 1; /* no protocol specified: safe */
+
+    if ( !isalpha(*p) )	/* protocol/method is [alpha][alnum or '+.-'] */
+	return 1;
+    while ( ++p < colon )
+	if ( !(isalnum(*p) || *p == '.' || *p == '+' || *p == '-') )
+	    return 1;
+
+    return isautoprefix(T(link), S(link));
+}
+
+
 /* print out a linky (or fail if it's Not Allowed)
  */
 static int
@@ -651,9 +684,7 @@ linkyformat(MMIOT *f, Cstring text, int image, Footnote *ref)
 	if ( f->flags & (MKD_NO_EXT|MKD_SAFELINK) )
 	    return 0;
     }
-    else if ( (f->flags & MKD_SAFELINK) && T(ref->link)
-				        && (T(ref->link)[0] != '/')
-				        && !isautoprefix(T(ref->link), S(ref->link)) )
+    else if ( (f->flags & MKD_SAFELINK) && !safelink(ref->link) )
 	/* if MKD_SAFELINK, only accept links that are local or
 	 * a well-known protocol
 	 */
@@ -1309,7 +1340,9 @@ text(MMIOT *f)
 		    break;
 	/* A^B -> A<sup>B</sup> */
 	case '^':   if ( (f->flags & (MKD_NOSUPERSCRIPT|MKD_STRICT|MKD_TAGTEXT))
-				|| (isthisnonword(f,-1) && peek(f,-1) != ')')
+				|| (f->last == 0)
+				|| ((ispunct(f->last) || isspace(f->last))
+				                        && f->last != ')')
 				|| isthisspace(f,1) )
 			Qchar(c,f);
 		    else {
@@ -1447,7 +1480,8 @@ text(MMIOT *f)
 		    }
 		    /* fall through to default */
 	
-	default:    Qchar(c, f);
+	default:    f->last = c;
+		    Qchar(c, f);
 		    break;
 	}
     }
@@ -1609,9 +1643,10 @@ printtable(Paragraph *pp, MMIOT *f)
 static int
 printblock(Paragraph *pp, MMIOT *f)
 {
-    Line *t = pp->text;
     static char *Begin[] = { "", "<p>", "<p style=\"text-align:center;\">"  };
     static char *End[]   = { "", "</p>","</p>" };
+    Line *t = pp->text;
+    int align = pp->align;
 
     while (t) {
 	if ( S(t->text) ) {
@@ -1631,9 +1666,9 @@ printblock(Paragraph *pp, MMIOT *f)
 	}
 	t = t->next;
     }
-    Qstring(Begin[pp->align], f);
+    Qstring(Begin[align], f);
     text(f);
-    Qstring(End[pp->align], f);
+    Qstring(End[align], f);
     return 1;
 }
 
@@ -1684,17 +1719,57 @@ printhtml(Line *t, MMIOT *f)
 
 
 static void
-htmlify(Paragraph *p, char *block, char *arguments, MMIOT *f)
+htmlify_paragraphs(Paragraph *p, MMIOT *f)
 {
     ___mkd_emblock(f);
-    if ( block )
-	Qprintf(f, arguments ? "<%s %s>" : "<%s>", block, arguments);
-    ___mkd_emblock(f);
 
     while (( p = display(p, f) )) {
 	___mkd_emblock(f);
 	Qstring("\n\n", f);
     }
+}
+
+
+#ifdef GITHUB_CHECKBOX
+static void
+li_htmlify(Paragraph *p, char *arguments, int flags, MMIOT *f)
+{
+    ___mkd_emblock(f);
+
+    Qprintf(f, "<li");
+    if ( arguments )
+	Qprintf(f, " %s", arguments);
+    if ( flags & GITHUB_CHECK )
+	Qprintf(f, " class=\"github_checkbox\"");
+    Qprintf(f, ">");
+#if CHECKBOX_AS_INPUT
+    if ( flags & GITHUB_CHECK ) {
+	Qprintf(f, "<input disabled=\"\" type=\"checkbox\"");
+	if ( flags & IS_CHECKED )
+	    Qprintf(f, " checked=\"checked\"");
+	Qprintf(f, "/>");
+    }
+#else
+    if ( flags & GITHUB_CHECK )
+	Qprintf(f, flags & IS_CHECKED ? "&#x2611;" : "&#x2610;");
+#endif
+
+    htmlify_paragraphs(p, f);
+
+     Qprintf(f, "</li>");
+    ___mkd_emblock(f);
+}
+#endif
+
+
+static void
+htmlify(Paragraph *p, char *block, char *arguments, MMIOT *f)
+{
+    ___mkd_emblock(f);
+    if ( block )
+	Qprintf(f, arguments ? "<%s %s>" : "<%s>", block, arguments);
+
+    htmlify_paragraphs(p, f);
 
     if ( block )
 	 Qprintf(f, "</%s>", block);
@@ -1736,7 +1811,11 @@ listdisplay(int typ, Paragraph *p, MMIOT* f)
 	Qprintf(f, ">\n");
 
 	for ( ; p ; p = p->next ) {
+#ifdef GITHUB_CHECKBOX
+	    li_htmlify(p->down, p->ident, p->flags, f);
+#else
 	    htmlify(p->down, "li", p->ident, f);
+#endif
 	    Qchar('\n', f);
 	}
 
@@ -1820,12 +1899,12 @@ mkd_extra_footnotes(MMIOT *m)
 	for ( j=0; j < S(m->footnotes->note); j++ ) {
 	    t = &T(m->footnotes->note)[j];
 	    if ( (t->refnumber == i) && (t->flags & REFERENCED) ) {
-		Csprintf(&m->out, "<li id=\"%s:%d\">\n<p>",
+		Csprintf(&m->out, "<li id=\"%s:%d\">\n",
 			    p_or_nothing(m), t->refnumber);
-		Csreparse(&m->out, T(t->title), S(t->title), 0);
+		htmlify(t->text, 0, 0, m);
 		Csprintf(&m->out, "<a href=\"#%sref:%d\" rev=\"footnote\">&#8617;</a>",
 			    p_or_nothing(m), t->refnumber);
-		Csprintf(&m->out, "</p></li>\n");
+		Csprintf(&m->out, "</li>\n");
 	    }
 	}
     }
@@ -1863,4 +1942,3 @@ mkd_document(Document *p, char **res)
     }
     return EOF;
 }
-
