@@ -170,8 +170,10 @@ splitline(Line *t, int cutpoint)
 	tmp->next = t->next;
 	t->next = tmp;
 
-	tmp->dle = t->dle;
 	SUFFIX(tmp->text, T(t->text)+cutpoint, S(t->text)-cutpoint);
+	EXPAND(tmp->text) = 0;
+	S(tmp->text)--;
+	
 	S(t->text) = cutpoint;
     }
 }
@@ -523,7 +525,7 @@ islist(Line *t, int *clip, DWORD flags, int *list_type)
 	i = nextnonblank(t, t->dle+1);
 	*clip = (i > 4) ? 4 : i;
 	*list_type = UL;
-	return AL;
+	return (flags & MKD_EXPLICITLIST) ? UL : AL;
     }
 
     if ( (j = nextblank(t,t->dle)) > t->dle ) {
@@ -608,8 +610,7 @@ codeblock(Paragraph *p)
     Line *t = p->text, *r;
 
     for ( ; t; t = r ) {
-	CLIP(t->text,0,4);
-	t->dle = mkd_firstnonblank(t);
+	__mkd_trim_line(t,4);
 
 	if ( !( (r = skipempty(t->next)) && iscode(r)) ) {
 	    ___mkd_freeLineRange(t,r);
@@ -636,6 +637,7 @@ iscodefence(Line *r, int size, line_type kind, DWORD flags)
 	return (r->kind == chk_tilde || r->kind == chk_backtick) && (r->count >= size);
 }
 
+
 static Paragraph *
 fencedcodeblock(ParagraphRoot *d, Line **ptr, DWORD flags)
 {
@@ -643,32 +645,32 @@ fencedcodeblock(ParagraphRoot *d, Line **ptr, DWORD flags)
     Paragraph *ret;
 
     first = (*ptr);
-    
+
     /* don't allow zero-length code fences
-     */
+    */
     if ( (first->next == 0) || iscodefence(first->next, first->count, 0, flags) )
 	return 0;
 
     /* find the closing fence, discard the fences,
-     * return a Paragraph with the contents
-     */
+    * return a Paragraph with the contents
+    */
     for ( r = first; r && r->next; r = r->next )
 	if ( iscodefence(r->next, first->count, first->kind, flags) ) {
 	    (*ptr) = r->next->next;
 	    ret = Pp(d, first->next, CODE);
-      if (S(first->text) - first->count > 0) {
-        char *lang_attr = T(first->text) + first->count;
-        while ( *lang_attr != 0 && *lang_attr == ' ' ) lang_attr++;
-        ret->lang = strdup(lang_attr);
-      }
-      else {
-        ret->lang = 0;
-      }
-	    ___mkd_freeLine(first);
-	    ___mkd_freeLine(r->next);
-	    r->next = 0;
-	    return ret;
+	    if (S(first->text) - first->count > 0) {
+		char *lang_attr = T(first->text) + first->count;
+		while ( *lang_attr != 0 && *lang_attr == ' ' ) lang_attr++;
+		ret->lang = strdup(lang_attr);
+	    }
+	    else {
+		ret->lang = 0;
 	}
+	___mkd_freeLine(first);
+	___mkd_freeLine(r->next);
+	r->next = 0;
+	return ret;
+    }
     return 0;
 }
 
@@ -803,9 +805,8 @@ quoteblock(Paragraph *p, DWORD flags)
 	    /* clip next space, if any */
 	    if ( T(t->text)[qp] == ' ' )
 		qp++;
-	    CLIP(t->text, 0, qp);
+	    __mkd_trim_line(t,qp);
 	    UNCHECK(t);
-	    t->dle = mkd_firstnonblank(t);
 	}
 
 	q = skipempty(t->next);
@@ -852,11 +853,35 @@ listitem(Paragraph *p, int indent, DWORD flags, linefn check)
     Line *t, *q;
     int clip = indent;
     int z;
+#ifdef GITHUB_CHECKBOX
+    int firstpara = 1;
+    int ischeck;
+#define CHECK_NOT 0
+#define CHECK_NO 1
+#define CHECK_YES 2
+#endif
 
     for ( t = p->text; t ; t = q) {
-	CLIP(t->text, 0, clip);
 	UNCHECK(t);
-	t->dle = mkd_firstnonblank(t);
+	__mkd_trim_line(t, clip);
+
+#ifdef GITHUB_CHECKBOX
+	if ( firstpara ) {
+	    ischeck = CHECK_NOT;
+	    if ( strncmp(T(t->text)+t->dle, "[ ]", 3) == 0 )
+		ischeck = CHECK_NO;
+	    else if ( strncasecmp(T(t->text)+t->dle, "[x]", 3) == 0 )
+		ischeck = CHECK_YES;
+
+	    if ( ischeck != CHECK_NOT ) {
+		__mkd_trim_line(t, 3);
+		p->flags |= GITHUB_CHECK;
+		if ( ischeck == CHECK_YES )
+		    p->flags |= IS_CHECKED;
+	    }
+	    firstpara = 0;
+	}
+#endif
 
         /* even though we had to trim a long leader off this item,
          * the indent for trailing paragraphs is still 4...
@@ -1005,6 +1030,28 @@ tgood(char c)
 }
 
 
+/*
+ * eat lines for a markdown extra footnote
+ */
+static Line *
+extrablock(Line *p)
+{
+    Line *np;
+    
+    while ( p && p->next ) {
+	np = p->next;
+
+	if ( np->dle < 4 && np->dle < S(np->text) ) {
+	    p->next = 0;
+	    return np;
+	}
+	__mkd_trim_line(np,4);
+	p = np;
+    }
+    return 0;
+}
+
+
 /*
  * add a new (image or link) footnote to the footnote table
  */
@@ -1020,20 +1067,31 @@ addfootnote(Line *p, MMIOT* f)
     CREATE(foot->tag);
     CREATE(foot->link);
     CREATE(foot->title);
+    foot->text = 0;
     foot->flags = foot->height = foot->width = 0;
 
+    /* keep the footnote label */
     for (j=i=p->dle+1; T(p->text)[j] != ']'; j++)
 	EXPAND(foot->tag) = T(p->text)[j];
-
     EXPAND(foot->tag) = 0;
     S(foot->tag)--;
+
+    /* consume the closing ]: */
     j = nextnonblank(p, j+2);
 
     if ( (f->flags & MKD_EXTRA_FOOTNOTE) && (T(foot->tag)[0] == '^') ) {
-	/* need to consume all lines until non-indented block? */
-	while ( j < S(p->text) )
-	    EXPAND(foot->title) = T(p->text)[j++];
-	goto skip_to_end;
+	/* markdown extra footnote: All indented lines past this point;
+	 * the first line includes the footnote reference, so we need to
+	 * snip that out as we go.
+	 */
+	foot->flags |= EXTRA_FOOTNOTE;
+	__mkd_trim_line(p,j);
+
+	np = extrablock(p);
+
+	foot->text = compile(p, 0, f);
+
+	return np;
     }
 
     while ( (j < S(p->text)) && !isspace(T(p->text)[j]) )
@@ -1044,8 +1102,7 @@ addfootnote(Line *p, MMIOT* f)
 
     if ( T(p->text)[j] == '=' ) {
 	sscanf(T(p->text)+j, "=%dx%d", &foot->width, &foot->height);
-	while ( (j < S(p->text)) && !isspace(T(p->text)[j]) )
-	    ++j;
+	j = nextblank(p, j);
 	j = nextnonblank(p,j);
     }
 
@@ -1075,7 +1132,6 @@ addfootnote(Line *p, MMIOT* f)
 	--S(foot->title);
     }
 
-skip_to_end:
     ___mkd_freeLine(p);
     return np;
 }
@@ -1249,6 +1305,7 @@ compile(Line *ptr, int toplevel, MMIOT *f)
     ptr = consume(ptr, &para);
 
     while ( ptr ) {
+
 	if ( iscode(ptr) ) {
 	    p = Pp(&d, ptr, CODE);
 	    
@@ -1290,13 +1347,39 @@ compile(Line *ptr, int toplevel, MMIOT *f)
 	    ptr = headerblock(p, hdr_type);
 	}
 	else {
-	    p = Pp(&d, ptr, MARKUP);
-	    ptr = textblock(p, toplevel, f->flags);
-	    /* tables are a special kind of paragraph */
-	    if ( actually_a_table(f, p->text) )
-		p->typ = TABLE;
-	}
+	    /* either markup or an html block element
+	     */
+	    struct kw *tag;
+	    int unclosed = 1;
 
+	    p = Pp(&d, ptr, MARKUP);	/* default to regular markup,
+					 * then check if it's an html
+					 * block.   If it IS an html
+					 * block, htmlblock() will
+					 * populate this paragraph &
+					 * all we need to do is reset
+					 * the paragraph type to HTML,
+					 * otherwise the paragraph
+					 * remains empty and ready for
+					 * processing with textblock()
+					 */
+	    
+	    if ( !(f->flags & MKD_NOHTML) && (tag = isopentag(ptr)) ) {
+		/* possibly an html block
+		 */
+		
+		ptr = htmlblock(p, tag, &unclosed);
+		if ( ! unclosed ) {
+		    p->typ = HTML;
+		}
+	    }
+	    if ( unclosed ) {
+		ptr = textblock(p, toplevel, f->flags);
+		/* tables are a special kind of paragraph */
+		if ( actually_a_table(f, p->text) )
+		    p->typ = TABLE;
+	    }
+	}
 	if ( (para||toplevel) && !p->align )
 	    p->align = PARA;
 
@@ -1329,9 +1412,10 @@ mkd_compile(Document *doc, DWORD flags)
     flags &= USER_FLAGS;
     
     if ( doc->compiled ) {
-	if ( doc->ctx->flags == flags )
+	if ( doc->ctx->flags == flags && !doc->dirty)
 	    return 1;
 	else {
+	    doc->compiled = doc->dirty = 0;
 	    if ( doc->code)
 		___mkd_freeParagraph(doc->code);
 	    if ( doc->ctx->footnotes )
