@@ -18,10 +18,6 @@
 #if defined(HAVE_BASENAME) && defined(HAVE_LIBGEN_H)
 #  include <libgen.h>
 #endif
-#if defined(HAVE_ALLOCA_H)
-#  include <alloca.h>
-#endif
-#include <unistd.h>
 #include <stdarg.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -32,13 +28,15 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <ctype.h>
+#include <unistd.h>
 
 #include "mkdio.h"
 #include "cstring.h"
 #include "amalloc.h"
+#include "gethopt.h"
 
 char *pgm = "theme";
-char *output = 0;
+char *output_file = 0;
 char *pagename = 0;
 char *root = 0;
 int   everywhere = 0;	/* expand all <?theme elements everywhere */
@@ -48,6 +46,11 @@ struct passwd *me = 0;
 #endif
 struct stat *infop = 0;
 
+#if USE_H1TITLE
+extern char* mkd_h1_title(MMIOT*);
+#endif
+
+extern int notspecial(char *filename);
 
 #define INTAG 0x01
 #define INHEAD 0x02
@@ -318,7 +321,14 @@ static void
 ftitle(MMIOT *doc, FILE* output, int flags, int whence)
 {
     char *h;
-    if ( (h = mkd_doc_title(doc)) == 0 && pagename )
+    h = mkd_doc_title(doc);
+
+#if USE_H1TITLE
+    if ( !h )
+	h = mkd_h1_title(doc);
+#endif
+
+    if ( !h )
 	h = pagename;
 
     if ( h )
@@ -503,60 +513,80 @@ spin(FILE *template, MMIOT *doc, FILE *output)
 } /* spin */
 
 
+struct h_opt opts[] = {
+    { 0, 0, 'c', "flags",  "set/show rendering options" },
+    { 0, 0, 'C', "bitmap", "set/show rendering options numerically" },
+    { 0, 0, 'd', "dir",    "set the document root" },
+    { 0, 0, 'E', 0,        "do all theme expansions everywhere" },
+    { 0, 0, 'f', 0,        "forcibly overwrite existing html files" },
+    { 0, 0, 'o', "file",   "write output to `file`" },
+    { 0, 0, 'p', "title",  "set the page title" },
+    { 0, 0, 't', "template",  "use `template` as template file" },
+    { 0, 0, 'V', 0,        "show version info" },
+} ;
+#define NROPTS (sizeof opts / sizeof opts[0])
+
 main(argc, argv)
 char **argv;
 {
     char *template = "page.theme";
     char *source = "stdin";
     FILE *tmplfile;
-    int opt;
     mkd_flag_t flags = THEME_CF|MKD_TOC;
     int force = 0;
     MMIOT *doc;
     struct stat sourceinfo;
+    char *q;
 
-    opterr=1;
+    struct h_opt *opt;
+    struct h_context blob;
+
+    hoptset(&blob, argc, argv);
+    hopterr(&blob, 1);
+    
     pgm = basename(argv[0]);
 
-    while ( (opt=getopt(argc, argv, "EfC:c:d:t:p:o:V")) != EOF ) {
-	switch (opt) {
-	case 'd':   root = optarg;
+    while ( opt = gethopt(&blob, opts, NROPTS) ) {
+	if ( opt == HOPTERR ) {
+	    hoptusage(pgm, opts, NROPTS, "[file]");
+	    exit(1);
+	}
+	switch ( opt->optchar ) {
+	case 'd':   root = hoptarg(&blob);
 		    break;
 	case 'E':   everywhere = 1;
 		    break;
-	case 'p':   pagename = optarg;
+	case 'p':   pagename = hoptarg(&blob);
 		    break;
 	case 'f':   force = 1;
 		    break;
-	case 't':   template = optarg;
+	case 't':   template = hoptarg(&blob);
 		    break;
-	case 'C':   if ( strcmp(optarg, "?") == 0 ) {
-			show_flags(0);
+	case 'C':   if ( strcmp(hoptarg(&blob), "?") == 0 ) {
+			show_flags(0,0);
 			exit(0);
 		    }
 		    else
-			flags = strtol(optarg, 0, 0);
+			flags = strtol(hoptarg(&blob), 0, 0);
 		    break;
-	case 'c':   if ( strcmp(optarg, "?") == 0 ) {
-			show_flags(1);
+	case 'c':   if ( strcmp(hoptarg(&blob), "?") == 0 ) {
+			show_flags(1,0);
 			exit(0);
 		    }
-		    else if ( !set_flag(&flags, optarg) )
-			fprintf(stderr,"%s: unknown option <%s>", pgm, optarg);
+		    else if ( q = set_flag(&flags, hoptarg(&blob)) )
+			fprintf(stderr,"%s: unknown option <%s>", pgm, q);
 		    break;		    
-	case 'o':   output = optarg;
+	case 'o':   output_file = hoptarg(&blob);
 		    break;
 	case 'V':   printf("theme+discount %s\n", markdown_version);
 		    exit(0);
-	default:    fprintf(stderr, "usage: %s [-V] [-d dir] [-p pagename] [-t template] [-o html] [file]\n", pgm);
-		    exit(1);
 	}
     }
 
     tmplfile = open_template(template);
 
-    argc -= optind;
-    argv += optind;
+    argc -= hoptind(&blob);
+    argv += hoptind(&blob);
 
 
     if ( argc > 0 ) {
@@ -576,30 +606,34 @@ char **argv;
 		fail("can't open either %s or %s", argv[0], source);
 	}
 
-	if ( !output ) {
+	if ( !output_file ) {
 	    char *p, *q;
-	    output = alloca(strlen(source) + strlen(".html") + 1);
 
-	    strcpy(output, source);
+	    
+	    if ( (output_file = malloc(strlen(source) + strlen(".html") + 1)) == 0 )
+		fail("out of memory allocating output file name buffer");
 
-	    if (( p = strchr(output, '/') ))
+	    strcpy(output_file, source);
+
+	    if (( p = strchr(output_file, '/') ))
 		q = strrchr(p+1, '.');
 	    else
-		q = strrchr(output, '.');
+		q = strrchr(output_file, '.');
 
 	    if ( q )
 		*q = 0;
 	    else
-		q = output + strlen(output);
+		q = output_file + strlen(output_file);
 
 	    strcat(q, ".html");
 	}
     }
-    if ( output && strcmp(output, "-") ) {
-	if ( force )
-	    unlink(output);
-	if ( !freopen(output, "w", stdout) )
-	    fail("can't write to %s", output);
+    if ( output_file && strcmp(output_file, "-") ) {
+	if ( force && notspecial(output_file) )
+	    unlink(output_file);
+	if ( !freopen(output_file, "w", stdout) ) {
+	    fail("can't write to %s", output_file);
+	}
     }
 
     if ( !pagename )
