@@ -38,7 +38,7 @@ mkd_toc(Document *p, char **doc)
     for ( tp = p->code; tp ; tp = tp->next ) {
 	if ( tp->typ == SOURCE ) {
 	    for ( srcp = tp->down; srcp; srcp = srcp->next ) {
-		if ( srcp->typ == HDR && srcp->text ) {
+		if ( (srcp->typ == HDR) && srcp->text ) {
 	    
 		    while ( last_hnumber > srcp->hnumber ) {
 			if ( (last_hnumber - srcp->hnumber) > 1 )
@@ -81,16 +81,12 @@ mkd_toc(Document *p, char **doc)
     }
 
     if ( (size = S(res)) > 0 ) {
+	/* null-terminate & strdup into a free()able memory chunk
+	 */
 	EXPAND(res) = 0;
-			/* HACK ALERT! HACK ALERT! HACK ALERT! */
-	*doc = T(res);  /* we know that a T(Cstring) is a character pointer
-			 * so we can simply pick it up and carry it away,
-			 * leaving the husk of the Ctring on the stack
-			 * END HACK ALERT
-			 */
+	*doc = strdup(T(res));
     }
-    else
-	DELETE(res);
+    DELETE(res);
     return size;
 }
 
